c**********************************************************************
c IO_INIT_PR.F
c**********************************************************************
c Read in P-R data and precalculate parameters for P-R acceleration
c
c Input:
c  infile	name of the thermal data input file
c  ntp		number of test particles
c
c Output:
c  in common block /prdrag/
c  pr(NTPMAX)		precomputed parameter of P-R drag
c
c Remarks: 
c Author: Miroslav Broz, miroslav.broz@email.cz
c Date: Feb 12th 2002

      subroutine io_init_pr(infile,ntp)

      include 'swift.inc'
      include 'pr.inc'
      include 'const.inc'

c  input
      character*(*) infile
      integer ntp

c  internal
      integer i,ierr,ntpchk
      real*8 R, rho

c  main
      write(*,*) 'P-R data file called ',infile
      call io_open(7,infile,'old','formatted',ierr)

      read(7,*,err=99,end=99) ntpchk
      if(ntp.eq.0) ntp = ntpchk
      if(ntpchk.ne.ntp) then
        write(*,*) ' SWIFT ERROR: in io_init_pr'
        write(*,*) '     Number of test particles differs in ',infile
        call util_exit(1)
      endif
 
      do i=1,ntpchk
        read(7,*,err=99,end=99) R, rho

        pr(i) = 3.d0/4.d0* S0/(cv**2*(rho*1.d3)*(R*1.d-6)) *day	! unit conversion !!!
      enddo

      close(unit = 7)
      write(*,*) ' '
      return

99    continue
      write(*,*) 'Error reading file',infile
      stop

      end    ! io_init_pr.f
c-----------------------------------------------------------------

